attach("IgusaInvariants.sage")
attach("TwistPolynomial12.sage")
attach("FrobeniusPolynomial.sage")

from sympy import factorint

def test_certifyGeometricallyIrreducible(a0,a1,a2,a3,a4,a5=1,a6=0):
	poly=ZZ['x']([a0,a1,a2,a3,a4,a5,a6])
	if poly.degree()==5 and poly.is_irreducible() :
		return 1;

	if test_CertifyCM(a0,a1,a2,a3,a4,a5,a6) :
		return 1;
	if split22(a0,a1,a2,a3,a4,a5,a6):
		return -1;
	if split33(a0,a1,a2,a3,a4,a5,a6):
		return -1;

	discPoly=poly.discriminant();
	maxP=103
	for p in range (5,maxP):
		if is_prime(p) and discPoly % p != 0 :
			q=Frobenius_Polynomial(p,a0,a1,a2,a3,a4,a5,a6)

			q=twistPoly12(q)
			if(q.coefficients(sparse=false)[2] % p != 0) :
	  			if(q.is_irreducible()) :
	    				return p;
	return -1;

def test_EndZ_Zarhin(a0,a1,a2,a3,a4,a5=1,a6=0):
	""" computes the Galois group of the polynomial. Returns true if it is S_n or A_n, false otherwise.
	"""
	p=a0+a1*x+a2*x^2+a3*x^3+a4*x^4+a5*x^5+a6*x^6
	q=ZZ['x']([a0,a1,a2,a3,a4,a5,a6])
	if q.is_irreducible() == false :
		return false
	K=NumberField(q,'a')
	G=K.galois_group(names='g', type='pari')
	if (a6==0 and G.order()>59) :
		return true
	else :
		if G.order()>359 :
			return true
		else:
			return false

def is_powerful(n):
	rad = factor(n).radical_value()
	return n % (rad^2) == 0

def test_EndZ_Hall(a0,a1,a2,a3,a4,a5=1,a6=0):
	'''
	"Hall's" test. The Jacobian has trivial geometric endomorphism ring if Jac(C) is geometrically irreducible and there is an odd prime that divides the discriminant of f(x) exactly once.
	'''
	q=ZZ['x']([a0,a1,a2,a3,a4,a5,a6])
	if q.is_irreducible() == false :
		return false

	'''
	Warning: the following assumes that Jac(C) has already been shown to be geometrically irreducible.
	To have a test that works in all scenarios, replace "return true" by "return (test_certifyGeometricallyIrreducible(a0,a1,a2,a3,a4,a5,a6)>0)"
	'''    
	if is_powerful(4*q.discriminant())== false:
		return true

	return false
