attach("Irreducibility.sage");
attach("Reducibility.sage");
attach("CM.sage");
attach("RM.sage");
attach("GeometricallyTrivial.sage");

def PrintOutput(bound, string,suffix="DefinitiveT"):
  f1=open('./Output'+suffix+str(bound), 'a')
  f1.write(string + "\n");
  print(string);
  f1.close();


def ComputeEndomorphismRingOfCurve(C,verbose=false):
	f,h=C.hyperelliptic_polynomials();
	ComputeEndomorphismRing(f,h,verbose);


def ComputeEndomorphismRing(f, h=0, verbose=false):
		if h==0 :
			poly=f;
		else :
			poly=4*f+h^2;
		if poly.degree()>6 or poly.degree() < 5:
			print("The curve does not have genus 2");
			return;
		coef=poly.coefficients(sparse=false);
		a0=coef[0];
		a1=coef[1];
		a2=coef[2];
		a3=coef[3];
		a4=coef[4];
		a5=coef[5];
		if poly.degree()==6 :
			a6=coef[6];
		else:
			a6=0;
		discPoly=poly.discriminant();
		if (discPoly != 0) : 
			isProvenIrreducible=(test_certifyGeometricallyIrreducible(a0,a1,a2,a3,a4,a5,a6)>0);
			if isProvenIrreducible:
				print("The Jacobian of the curve is geometrically irreducible");
				if(test_EndZ_Hall(a0,a1,a2,a3,a4,a5,a6)==false) :
					if (test_EndZ_Zarhin(a0,a1,a2,a3,a4,a5,a6) == false) :
						if CertifyEndZ(a0,a1,a2,a3,a4,a5,a6):
							print("The Jacobian of the curve has trivial geometric endomorphism ring");
						else:
							if test_CertifyCM(a0,a1,a2,a3,a4,a5,a6,verbose):
								print("The Jacobian of the curve admits Complex Multiplication");
							else :
								RMDiscriminant=CertifyRM(a0,a1,a2,a3,a4,a5,a6,verbose);
								if RMDiscriminant>0 :
									print("The Jacobian of the curve has geometric RM by a quadratic ring of discriminant "+str(RMDiscriminant) + ".");
								else :
									print("The Jacobian of the curve is likely to have geometric RM, but this statement cannot be proven with the current limits of this algorithm.");
									print("Bound on the square of the discriminant of the endomorphism ring: "+str(BoundDiscriminant(a0,a1,a2,a3,a4,a5,a6)));
					else:
						print("The Jacobian of the curve satisfies Zarhin's condition, so its geometric endomorphism ring is trivial");
				else:
					print("The Jacobian of the curve has a place of semistable reduction of toric rank 1, so its geometric endomorphism ring is trivial");
			else:
				print("Cannot (yet) prove that the Jacobian of the curve is geometrically irreducible. Testing for splitting or quaternionic multiplication.");
				SplittingOrder=test_Reducibility(a0,a1,a2,a3,a4,a5,a6,verbose);
				if SplittingOrder>0 :
					print("The Jacobian of the curve is geometrically (" + str(SplittingOrder) + "," + str(SplittingOrder) + ")-split.");
				else :
					if test_certifyNonQM(a0,a1,a2,a3,a4,a5,a6)==true :
						print("The Jacobian of the curve does not admit geometric QM. It is likely to be geometrically split (of order at least 6), but this statement cannot be proven with the current limits of this algorithm.");
					else:
						QMResult=test_certifyQM(a0,a1,a2,a3,a4,a5,a6);
						if(QMResult!=false ):
							print("The Jacobian of the curve is geometrically simple, and has QM by the quaternionic ring corresponding to the matrix:\n" + str(QMResult));
						else:
							print("The Jacobian of the curve is likely to have Quaternionic Multiplication, but this statement cannot be proven with the current limits of this algorithm.");

		else :
			print("The curve is singular");
			singular=singular+1;

